@@ -87,6 +87,7 @@ int pass;		/* how many times have we re-polled? */
 int stage;		/* where are we? */
 int phase;		/* where are we, for error-logging purposes? */
 int mytimeout;		/* value of nonreponse timeout */
+int suppress_tags;	/* emit tags? */
 
 static const struct method *protocol;
 static jmp_buf	restart;
@@ -531,6 +532,21 @@ static int readheaders(int sock,
 		goto process_headers;
 	    }
 
+	    /*
+	     * At least one brain-dead website (netmind.com) is known to
+	     * send out robotmail that's missing the RFC822 delimiter blank
+	     * line before the body! Without this check fetchmail segfaults.
+	     * With it, we treat such messages as though they had the missing
+	     * blank line.
+	     */
+	    if (!isspace(line[0]) && !strchr(line, ':'))
+	    {
+		headers_ok = TRUE;
+		free(line);
+		has_nuls = (linelen != strlen(line));
+		goto process_headers;
+	    }
+
 	    /* check for RFC822 continuations */
 	    set_timeout(mytimeout);
 	    ch = SockPeek(sock);
@@ -654,13 +670,14 @@ static int readheaders(int sock,
 	 * 
 	 * This is to avoid false mail loops messages when delivering
 	 * local messages to and from a Postfix/qmail mailserver. 
-	 * 
-	 * Should be controlled by an option
 	 */
-	if (ctl->dropdelivered && !strncasecmp(line, "Delivered-To:", 13)) {
-      if (delivered_to) free(line);
-      else delivered_to = line;
-	  continue;
+	if (ctl->dropdelivered && !strncasecmp(line, "Delivered-To:", 13)) 
+	{
+	    if (delivered_to)
+		free(line);
+	    else 
+		delivered_to = line;
+	    continue;
 	}
 
 	/*
@@ -740,14 +757,16 @@ static int readheaders(int sock,
 	}
 	else
 	{
+	    char *newhdrs;
 	    int	newlen;
 
 	    newlen = oldlen + strlen(line);
-	    msgblk.headers = (char *) realloc(msgblk.headers, newlen + 1);
-	    if (msgblk.headers == NULL) {
+	    newhdrs = (char *) realloc(msgblk.headers, newlen + 1);
+	    if (newhdrs == NULL) {
 		free(line);
 		return(PS_IOERR);
 	    }
+	    msgblk.headers = newhdrs;
 	    strcpy(msgblk.headers + oldlen, line);
 	    free(line);
 	    line = msgblk.headers + oldlen;
@@ -1291,11 +1310,7 @@ int
 kerberos_auth (socket, canonical, principal) 
 /* authenticate to the server host using Kerberos V4 */
 int socket;		/* socket to server host */
-#if defined(__FreeBSD__) || defined(__OpenBSD__)
 char *canonical;	/* server name */
-#else
-const char *canonical;	/* server name */
-#endif
 char *principal;
 {
     char * host_primary;
@@ -1816,9 +1831,12 @@ const int maxfetch;		/* maximum number of messages to fetch */
 		     * calling user a heads-up about the authentication 
 		     * failure once it looks like this isn't a fluke 
 		     * due to the server being temporarily inaccessible.
+		     * When we get third failure, we notify the user.  After
+		     * that, once we get authorization we let the user know
+		     * service is restored.
 		     */
 		    if (run.poll_interval
-			&& ctl->authfailcount++ > MAX_AUTHFAILS 
+			&& ++ctl->authfailcount == 3
 			&& !open_warning_by_mail(ctl, (struct msgblk *)NULL))
 		    {
 			stuff_warning(ctl,
@@ -1827,13 +1845,16 @@ const int maxfetch;		/* maximum number of messages to fetch */
 			    _("Fetchmail could not get mail from %s@%s.\r\n"), 
 			    ctl->remotename,
 			    ctl->server.truename);
-			stuff_warning(ctl, 
-    _("The attempt to get authorization failed.\r\n" \
-    "This probably means your password is invalid, but POP3 servers have\r\n" \
-    "other failure modes that fetchmail cannot distinguish from this\r\n" \
-    "because they don't send useful error messages on login failure.\r\n"));
+			stuff_warning(ctl, _("\
+The attempt to get authorization failed.\r\n\
+This probably means your password is invalid, but some servers have\r\n\
+other failure modes that fetchmail cannot distinguish from this\r\n\
+because they don't send useful error messages on login failure.\r\n\
+\r\n\
+The fetchmail daemon will continue running and attempt to connect\r\n\
+at each cycle.  No future notifications will be sent until service\r\n\
+is restored."));
 			close_warning_by_mail(ctl, (struct msgblk *)NULL);
-			ctl->wedged = TRUE;
 		    }
 		}
 		else
@@ -1843,6 +1864,30 @@ const int maxfetch;		/* maximum number of messages to fetch */
 		    
 		goto cleanUp;
 	    }
+	    else
+	    {
+		if (ctl->authfailcount >= 3)
+		{
+		    report(stderr,
+			   _("Authorization OK on %s@%s\n"),
+			   ctl->remotename,
+			   ctl->server.truename);
+		    if (!open_warning_by_mail(ctl, (struct msgblk *)NULL))
+		    {
+			stuff_warning(ctl,
+			    _("Subject: fetchmail authentication OK\r\n"));
+			stuff_warning(ctl,
+			    _("Fetchmail was able to log into %s@%s.\r\n"), 
+			    ctl->remotename,
+			    ctl->server.truename);
+			stuff_warning(ctl, 
+			    _("Service has been restored.\r\n"));
+			close_warning_by_mail(ctl, (struct msgblk *)NULL);
+		    
+		    }
+		    ctl->authfailcount = 0;
+		}
+	    }
 	}
 
 	ctl->errcount = fetches = 0;
@@ -2342,9 +2387,6 @@ const int maxfetch;		/* maximum number of messages to fetch */
     case PS_SOCKET:
 	msg = _("socket");
 	break;
-    case PS_AUTHFAIL:
-	msg = _("authorization");
-	break;
     case PS_SYNTAX:
 	msg = _("missing or bad RFC822 header");
 	break;
@@ -2370,8 +2412,8 @@ const int maxfetch;		/* maximum number of messages to fetch */
 	report(stderr, _("undefined error\n"));
 	break;
     }
-    /* no report on PS_MAXFETCH or PS_UNDEFINED */
-    if (ok==PS_SOCKET || ok==PS_AUTHFAIL || ok==PS_SYNTAX 
+    /* no report on PS_MAXFETCH or PS_UNDEFINED or PS_AUTHFAIL */
+    if (ok==PS_SOCKET || ok==PS_SYNTAX
 		|| ok==PS_IOERR || ok==PS_ERROR || ok==PS_PROTOCOL 
 		|| ok==PS_LOCKBUSY || ok==PS_SMTP || ok==PS_DNS)
 	report(stderr, _("%s error while fetching from %s\n"), msg, ctl->server.pollname);
@@ -2496,7 +2538,7 @@ va_dcl
     char buf [MSGBUFSIZE+1];
     va_list ap;
 
-    if (protocol->tagged)
+    if (protocol->tagged && !suppress_tags)
 	(void) sprintf(buf, "%s ", GENSYM);
     else
 	buf[0] = '\0';
@@ -2582,7 +2624,7 @@ va_dcl
 
     phase = SERVER_WAIT;
 
-    if (protocol->tagged)
+    if (protocol->tagged && !suppress_tags)
 	(void) sprintf(buf, "%s ", GENSYM);
     else
 	buf[0] = '\0';
