@@ -12,11 +12,8 @@
 #define		P_APOP		4
 #define		P_RPOP		5
 #define		P_IMAP		6
-#define		P_IMAP_K4	7
-#define		P_IMAP_GSS	8
-#define		P_IMAP_CRAM_MD5	9
-#define		P_IMAP_LOGIN	10
-#define		P_ETRN		11
+#define		P_ETRN		7
+#define		P_ODMR		8
 
 #if INET6_ENABLE
 #define		SMTP_PORT	"smtp"
@@ -32,10 +29,12 @@
 #endif
 
 /* preauthentication types */
-#define		A_PASSWORD	0	/* password or inline authentication */
-#define		A_KERBEROS_V4	1	/* preauthenticate w/ Kerberos V4 */
-#define		A_KERBEROS_V5	2	/* preauthenticate w/ Kerberos V5 */
-#define		A_SSH		3	/* preauthentication at session level */
+#define		A_ANY		0	/* use the first method that works */
+#define		A_PASSWORD	1	/* password or inline authentication */
+#define		A_KERBEROS_V4	2	/* preauthenticate w/ Kerberos V4 */
+#define		A_KERBEROS_V5	3	/* preauthenticate w/ Kerberos V5 */
+#define 	A_GSSAPI	4	/* preauthenticate with GSSAPI */
+#define		A_SSH		5	/* preauthentication at session level */
 
 /*
  * Definitions for buffer sizes.  We get little help on setting maxima
@@ -115,6 +114,7 @@ struct runctl
     int		poll_interval;
     char	*postmaster;
     flag	bouncemail;
+    flag	spambounce;
     char	*properties;
     flag	use_syslog;
     flag	invisible;
@@ -361,6 +361,7 @@ extern int pass;		/* number of re-polling pass */
 extern flag configdump;		/* dump control blocks as Python dictionary */
 extern const char *fetchmailhost;
 				/* either "localhost" or an FQDN */
+extern suppress_tags;		/* suppress tags in tagged protocols? */
 #ifdef SDPS_ENABLE
 extern char *sdps_envfrom;
 extern char *sdps_envto;
@@ -515,11 +516,18 @@ int doPOP3 (struct query *);
 int doIMAP (struct query *);
 int doETRN (struct query *);
 
+/* authentication functions */
+int do_cram_md5(int sock, char *command, struct query *ctl);
+int do_rfc1731(int sock, char *command, char *truename);
+int do_gssauth(int sock, char *command, char *hostname, char *username);
+int do_otp(int sock, char *command, struct query *ctl);
+
 /* miscellanea */
 struct query *hostalloc(struct query *); 
 int parsecmdline (int, char **, struct runctl *, struct query *);
 char *MD5Digest (unsigned char *);
 int POP3_auth_rpa(unsigned char *, unsigned char *, int socket);
+void deal_with_sigchld(void);
 int daemonize(const char *, void (*)(int));
 char *fm_getpassword(char *);
 void escapes(const char *, char *);
