@@ -198,6 +198,68 @@ int pop3_getauth(int sock, struct query *ctl, char *greeting)
 	}
 #endif /* OPIE_ENABLE */
 
+	/*
+	 * AUTH command may return a list of available mechanisms.
+	 * if it doesn't, no harm done.  Efficiency hack: most servers
+	 * don't implement this, so don't do it at all unless the
+	 * server advertises APOP with <> in the greeting line.  This
+	 * certainly catches IMAP-2000's POP3 gateway.
+	 * 
+	 * These authentication methods are blessed by RFC1734,
+	 * POP3 AUTHentication command.
+	 */
+	if (strchr(greeting, '<') && gen_transact(sock, "AUTH") == 0)
+	{
+	    char buffer[10];
+#if defined(GSSAPI)
+	    flag has_gssapi = FALSE;
+#endif /* defined(GSSAPI) */
+#if defined(KERBEROS_V4) || defined(KERBEROS_V5)
+	    flag has_kerberos = FALSE;
+#endif /* defined(KERBEROS_V4) || defined(KERBEROS_V5) */
+	    flag has_cram = FALSE;
+#ifdef OPIE_ENABLE
+	    flag has_opie = FALSE;
+#endif /* OPIE_ENABLE */
+
+	    /* determine what authentication methods we have available */
+	    while ((ok = gen_recv(sock, buffer, sizeof(buffer))) == 0)
+	    {
+		if (DOTLINE(buffer))
+		    break;
+#if defined(GSSAPI)
+		if (strncasecmp(buffer, "GSSAPI", 8) == 0)
+		    has_gssapi = TRUE;
+#endif /* defined(GSSAPI) */
+#if defined(KERBEROS_V4) || defined(KERBEROS_V5)
+		if (strncasecmp(buffer, "KERBEROS_V4", 8) == 0)
+		    has_kerberos = TRUE;
+#endif /* defined(KERBEROS_V4) || defined(KERBEROS_V5) */
+		if (strncasecmp(buffer, "CRAM-MD5", 8) == 0)
+		    has_cram = TRUE;
+#ifdef OPIE_ENABLE
+		if (strncasecmp(buffer, "X-OTP", 8) == 0)
+		    has_opie = TRUE;
+#endif /* OPIE_ENABLE */
+	    }
+
+#if defined(GSSAPI)
+	    if (has_kerberos)
+		return(do_gssauth(sock, "AUTH", 
+				  ctl->server.truename, ctl->remotename));
+#endif /* defined(GSSAPI) */
+#if defined(KERBEROS_V4) || defined(KERBEROS_V5)
+	    if (has_kerberos)
+		return(do_rfc1731(sock, "AUTH", ctl->server.truename));
+#endif /* defined(KERBEROS_V4) || defined(KERBEROS_V5) */
+	    if (has_cram)
+		return(do_cram_md5(sock, "AUTH", ctl));
+#ifdef OPIE_ENABLE
+	    if (has_opie)
+		do_otp(sock, "AUTH", ctl)
+#endif /* OPIE_ENABLE */
+       }
+
 	/* ordinary validation, no one-time password or RPA */ 
 	ok = gen_transact(sock, "PASS %s", ctl->password);
 	break;
